@@ -27,6 +27,8 @@
  *******************************************************************************/
 package org.sat4j.tools;
 
+import java.math.BigInteger;
+
 import org.sat4j.core.Vec;
 import org.sat4j.core.VecInt;
 import org.sat4j.specs.ContradictionException;
@@ -276,6 +278,25 @@ public class GateTranslator extends SolverDecorator<ISolver> {
 		return constrs;
 	}
 
+	/**
+	 * @since 2.2
+	 */
+	public void xor(int x, int a, int b) throws ContradictionException {
+		IVecInt clause = new VecInt(3);
+		clause.push(-a).push(b).push(x);
+		processClause(clause);
+		clause.clear();
+		clause.push(a).push(-b).push(x);
+		processClause(clause);
+		clause.clear();
+		clause.push(-a).push(-b).push(-x);
+		processClause(clause);
+		clause.clear();
+		clause.push(a).push(b).push(-x);
+		processClause(clause);
+		clause.clear();
+	}
+
 	private void xor2Clause(int[] f, int prefix, boolean negation,
 			IVec<IConstr> constrs) throws ContradictionException {
 		if (prefix == f.length - 1) {
@@ -328,4 +349,180 @@ public class GateTranslator extends SolverDecorator<ISolver> {
 		}
 	}
 
+	// From Een and Sorensson JSAT 2006 paper
+
+	/**
+	 * @since 2.2
+	 */
+	public void fullAdderSum(int x, int a, int b, int c)
+			throws ContradictionException {
+		IVecInt clause = new VecInt(4);
+		// -a /\ -b /\ -c -> -x
+		clause.push(a).push(b).push(c).push(-x);
+		processClause(clause);
+		clause.clear();
+		// -a /\ b /\ c -> -x
+		clause.push(a).push(-b).push(-c).push(-x);
+		processClause(clause);
+		clause.clear();
+		clause.push(-a).push(b).push(-c).push(-x);
+		processClause(clause);
+		clause.clear();
+		clause.push(-a).push(-b).push(c).push(-x);
+		processClause(clause);
+		clause.clear();
+		clause.push(-a).push(-b).push(-c).push(x);
+		processClause(clause);
+		clause.clear();
+		clause.push(-a).push(b).push(c).push(x);
+		processClause(clause);
+		clause.clear();
+		clause.push(a).push(-b).push(c).push(x);
+		processClause(clause);
+		clause.clear();
+		clause.push(a).push(b).push(-c).push(x);
+		processClause(clause);
+		clause.clear();
+	}
+
+	/**
+	 * @since 2.2
+	 */
+	public void fullAdderCarry(int x, int a, int b, int c)
+			throws ContradictionException {
+		IVecInt clause = new VecInt(3);
+		clause.push(-b).push(-c).push(x);
+		processClause(clause);
+		clause.clear();
+		clause.push(-a).push(-c).push(x);
+		processClause(clause);
+		clause.clear();
+		clause.push(-a).push(-b).push(x);
+		processClause(clause);
+		clause.clear();
+		clause.push(b).push(c).push(-x);
+		processClause(clause);
+		clause.clear();
+		clause.push(a).push(c).push(-x);
+		processClause(clause);
+		clause.clear();
+		clause.push(a).push(b).push(-x);
+		processClause(clause);
+		clause.clear();
+	}
+
+	/**
+	 * @since 2.2
+	 */
+	public void additionalFullAdderConstraints(int xcarry, int xsum, int a,
+			int b, int c) throws ContradictionException {
+		IVecInt clause = new VecInt(3);
+		clause.push(-xcarry).push(-xsum).push(a);
+		processClause(clause);
+		clause.push(-xcarry).push(-xsum).push(b);
+		processClause(clause);
+		clause.push(-xcarry).push(-xsum).push(c);
+		processClause(clause);
+		clause.push(xcarry).push(xsum).push(-a);
+		processClause(clause);
+		clause.push(xcarry).push(xsum).push(-b);
+		processClause(clause);
+		clause.push(xcarry).push(xsum).push(-c);
+		processClause(clause);
+	}
+
+	/**
+	 * @since 2.2
+	 */
+	public void halfAdderSum(int x, int a, int b) throws ContradictionException {
+		xor(x, a, b);
+	}
+
+	/**
+	 * @since 2.2
+	 */
+	public void halfAdderCarry(int x, int a, int b)
+			throws ContradictionException {
+		and(x, a, b);
+	}
+
+	/**
+	 * Translate an optimization function into constraints and provides the
+	 * binary literals in results. Works only when the value of the objective
+	 * function is positive.
+	 * 
+	 * @since 2.2
+	 */
+	public void optimisationFunction(IVecInt literals, IVec<BigInteger> coefs,
+			IVecInt result) throws ContradictionException {
+		IVec<IVecInt> buckets = new Vec<IVecInt>();
+		IVecInt bucket;
+		// filling the buckets
+		for (int i = 0; i < literals.size(); i++) {
+			int p = literals.get(i);
+			BigInteger a = coefs.get(i);
+			for (int j = 0; j < a.bitLength(); j++) {
+				bucket = createIfNull(buckets, j);
+				if (a.testBit(j)) {
+					bucket.push(p);
+				}
+			}
+		}
+		// creating the adder
+		int x, y, z;
+		int sum, carry;
+		for (int i = 0; i < buckets.size(); i++) {
+			bucket = buckets.get(i);
+			while (bucket.size() >= 3) {
+				x = bucket.get(0);
+				y = bucket.get(1);
+				z = bucket.get(2);
+				bucket.remove(x);
+				bucket.remove(y);
+				bucket.remove(z);
+				sum = nextFreeVarId(true);
+				carry = nextFreeVarId(true);
+				fullAdderSum(sum, x, y, z);
+				fullAdderCarry(carry, x, y, z);
+				additionalFullAdderConstraints(carry, sum, x, y, z);
+				bucket.push(sum);
+				createIfNull(buckets, i + 1).push(carry);
+			}
+			while (bucket.size() == 2) {
+				x = bucket.get(0);
+				y = bucket.get(1);
+				bucket.remove(x);
+				bucket.remove(y);
+				sum = nextFreeVarId(true);
+				carry = nextFreeVarId(true);
+				halfAdderSum(sum, x, y);
+				halfAdderCarry(carry, x, y);
+				bucket.push(sum);
+				createIfNull(buckets, i + 1).push(carry);
+			}
+			assert bucket.size() == 1;
+			result.push(bucket.last());
+			bucket.pop();
+			assert bucket.isEmpty();
+		}
+	}
+
+	/**
+	 * Create a new bucket if it does not exists. Works only because the buckets
+	 * are explored in increasing order.
+	 * 
+	 * @param buckets
+	 * @param i
+	 * @return
+	 */
+	private IVecInt createIfNull(IVec<IVecInt> buckets, int i) {
+		IVecInt bucket = buckets.get(i);
+		if (bucket == null) {
+			bucket = new VecInt();
+			buckets.push(bucket);
+			assert buckets.get(i) == bucket;
+		}
+		return bucket;
+
+	}
 }
