@@ -52,6 +52,8 @@ public class OptToPBSATAdapter extends PBSolverDecorator {
 
 	private final IVecInt assumps = new VecInt();
 
+	private long begin;
+
 	public OptToPBSATAdapter(IOptimizationProblem problem) {
 		super((IPBSolver) problem);
 		this.problem = problem;
@@ -61,6 +63,7 @@ public class OptToPBSATAdapter extends PBSolverDecorator {
 	public boolean isSatisfiable() throws TimeoutException {
 		modelComputed = false;
 		assumps.clear();
+		begin = System.currentTimeMillis();
 		if (problem.hasNoObjectiveFunction()) {
 			return modelComputed = problem.isSatisfiable();
 		}
@@ -73,20 +76,21 @@ public class OptToPBSATAdapter extends PBSolverDecorator {
 	}
 
 	@Override
-	public boolean isSatisfiable(IVecInt assumps, boolean global)
+	public boolean isSatisfiable(IVecInt myAssumps, boolean global)
 			throws TimeoutException {
-		return isSatisfiable(assumps);
+		return isSatisfiable(myAssumps);
 	}
 
 	@Override
-	public boolean isSatisfiable(IVecInt assumps) throws TimeoutException {
+	public boolean isSatisfiable(IVecInt myAssumps) throws TimeoutException {
 		modelComputed = false;
 		this.assumps.clear();
-		assumps.copyTo(this.assumps);
+		myAssumps.copyTo(this.assumps);
+		begin = System.currentTimeMillis();
 		if (problem.hasNoObjectiveFunction()) {
-			return modelComputed = problem.isSatisfiable(assumps);
+			return modelComputed = problem.isSatisfiable(myAssumps);
 		}
-		return problem.admitABetterSolution(assumps);
+		return problem.admitABetterSolution(myAssumps);
 	}
 
 	@Override
@@ -98,6 +102,13 @@ public class OptToPBSATAdapter extends PBSolverDecorator {
 			assert !problem.hasNoObjectiveFunction();
 			do {
 				problem.discardCurrentSolution();
+				if (isVerbose()) {
+					System.out.println(getLogPrefix()
+							+ "Current objective function value: "
+							+ problem.getObjectiveValue() + "("
+							+ ((System.currentTimeMillis() - begin) / 1000.0)
+							+ "s)");
+				}
 			} while (problem.admitABetterSolution(assumps));
 		} catch (TimeoutException e) {
 			// solver timeout
