@@ -92,20 +92,20 @@ public class OPBReader2005 extends Reader implements Serializable {
 
 	protected final IPBSolver solver;
 
-	private final IVecInt lits;
+	protected final IVecInt lits;
 
-	private final IVec<BigInteger> coeffs;
+	protected final IVec<BigInteger> coeffs;
 
-	private BigInteger d;
+	protected BigInteger d;
 
-	private String operator;
+	protected String operator;
 
 	private final IVecInt objectiveVars = new VecInt();
 
 	private final IVec<BigInteger> objectiveCoeffs = new Vec<BigInteger>();
 
 	// does the instance have an objective function?
-	private boolean hasObjFunc = false;
+	protected boolean hasObjFunc = false;
 
 	// does the instance need variables explanation?
 	protected boolean hasVariablesExplanation = false;
@@ -184,13 +184,15 @@ public class OPBReader2005 extends Reader implements Serializable {
 	 *            the coefficient of the term
 	 * @param var
 	 *            the identifier of the variable
+	 * @throws ParseFormatException
 	 */
-	private void constraintTerm(BigInteger coeff, String var) {
+	private void constraintTerm(BigInteger coeff, String var)
+			throws ParseFormatException {
 		coeffs.push(coeff);
 		lits.push(translateVarToId(var));
 	}
 
-	protected int translateVarToId(String var) {
+	protected int translateVarToId(String var) throws ParseFormatException {
 		int id = Integer.parseInt(var.substring(1));
 		return ((savedChar == '-') ? -1 : 1) * id;
 	}
@@ -285,8 +287,9 @@ public class OPBReader2005 extends Reader implements Serializable {
 	protected void skipSpaces() throws IOException {
 		char c;
 
-		while (Character.isWhitespace(c = get()))
-			;
+		do {
+			c = get();
+		} while (Character.isWhitespace(c));
 
 		putback(c);
 	}
@@ -504,8 +507,8 @@ public class OPBReader2005 extends Reader implements Serializable {
 	}
 
 	/**
-	 * @throws IOException 
-	 * @throws ParseFormatException  
+	 * @throws IOException
+	 * @throws ParseFormatException
 	 */
 	protected void readVariablesExplanation() throws IOException,
 			ParseFormatException {
@@ -518,7 +521,7 @@ public class OPBReader2005 extends Reader implements Serializable {
 	 * @throws IOException
 	 * @throws ParseException
 	 */
-	private void readObjective() throws IOException, ParseFormatException {
+	protected void readObjective() throws IOException, ParseFormatException {
 		char c;
 		StringBuffer var = new StringBuffer();
 		StringBuffer coeff = new StringBuffer();
@@ -566,7 +569,7 @@ public class OPBReader2005 extends Reader implements Serializable {
 	 * @throws IOException
 	 * @throws ContradictionException
 	 */
-	private void readConstraint() throws IOException, ParseFormatException,
+	protected void readConstraint() throws IOException, ParseFormatException,
 			ContradictionException {
 		StringBuffer var = new StringBuffer();
 		StringBuffer coeff = new StringBuffer();
@@ -660,13 +663,14 @@ public class OPBReader2005 extends Reader implements Serializable {
 		}
 		// Small check on the number of constraints
 		if (nbConstraintsRead != nbConstr) {
-			throw new ParseFormatException(
-					"Number of constraints read ("+nbConstraintsRead+") is different from metadata ("+nbConstr+")");
+			throw new ParseFormatException("Number of constraints read ("
+					+ nbConstraintsRead + ") is different from metadata ("
+					+ nbConstr + ")");
 		}
 	}
 
 	@Override
-	public final IProblem parseInstance(final java.io.Reader input)
+	public IProblem parseInstance(final java.io.Reader input)
 			throws ParseFormatException, ContradictionException {
 		IProblem problem = parseInstance(new LineNumberReader(input));
 		solver.setObjectiveFunction(getObjectiveFunction());
@@ -680,8 +684,20 @@ public class OPBReader2005 extends Reader implements Serializable {
 		try {
 			parse();
 			return solver;
-		} catch (IOException e) {
-			throw new ParseFormatException(e);
+                } catch (ContradictionException ce) {
+                        throw ce;
+		} catch (Exception e) {
+			String message;
+
+			if (e instanceof ParseFormatException) {
+				message = e.getMessage().substring(
+						"DIMACS Format error: ".length());
+			} else {
+				message = e.getMessage();
+			}
+			throw new ParseFormatException(" line " + input.getLineNumber()
+					+ ", " + message);
+
 		}
 	}
 
