@@ -52,6 +52,7 @@
 package org.sat4j.pb.reader;
 
 import java.io.IOException;
+import java.io.PrintWriter;
 import java.math.BigInteger;
 import java.util.HashMap;
 import java.util.Iterator;
@@ -113,7 +114,7 @@ public class OPBReader2007 extends OPBReader2006 {
 	/**
 	 * contains the number of new symbols generated to linearize products
 	 */
-	private int nbNewSymbols;
+	protected int nbNewSymbols;
 
 	@Override
 	protected void readTerm(StringBuffer coeff, StringBuffer var)
@@ -153,11 +154,16 @@ public class OPBReader2007 extends OPBReader2006 {
 	 * @param lits
 	 *            a set of literals in DIMACS format in which var once
 	 *            translated will be added.
+	 * @throws ParseFormatException
 	 */
-	protected void literalInAProduct(String var, IVecInt lits) {
+	protected void literalInAProduct(String var, IVecInt lits)
+			throws ParseFormatException {
 		int beginning = ((var.charAt(0) == '~') ? 2 : 1);
 		int id = Integer.parseInt(var.substring(beginning));
 		int lid = ((var.charAt(0) == '~') ? -1 : 1) * id;
+		if (lid == 0 || Math.abs(lid) >= nbNewSymbols) {
+			throw new ParseFormatException("Wrong variable id");
+		}
 		lits.push(lid);
 	}
 
@@ -228,24 +234,36 @@ public class OPBReader2007 extends OPBReader2006 {
 	}
 
 	@Override
-	protected int translateVarToId(String var) {
+	protected int translateVarToId(String var) throws ParseFormatException {
 		int beginning = ((var.charAt(0) == '~') ? 2 : 1);
 		int id = Integer.parseInt(var.substring(beginning));
+		if (id == 0 || id >= nbNewSymbols) {
+			throw new ParseFormatException("Wrong variable id format: " + var);
+		}
 		return ((var.charAt(0) == '~') ? -1 : 1) * id;
 	}
 
 	private String linearizeProduct(IVec<String> tmpLit)
-			throws ContradictionException {
+			throws ContradictionException, ParseFormatException {
 		tmpLit.sort(String.CASE_INSENSITIVE_ORDER);
 		String newVar = getProductVariable(tmpLit);
 		if (newVar == null) {
 			// generate a new symbol
-			newVar = "X" + String.valueOf(nbNewSymbols++);
+			newVar = "X" + nbNewSymbols++;
 			// linearization proposed by O. Roussel (PB07)
 			// generate the clause
 			// product => newSymbol (this is a clause)
 			// not x1 or not x2 ... or not xn or newSymbol
-			productStore.put(newVar, tmpLit);
+			if (tmpLit.size() == 2) {
+				Map<String, String> map1 = binaryProductToVar
+						.get(tmpLit.get(0));
+				if (map1 == null) {
+					map1 = new HashMap<String, String>();
+					binaryProductToVar.put(tmpLit.get(0), map1);
+				}
+				map1.put(tmpLit.get(1), newVar);
+			}
+			varToProduct.put(newVar, tmpLit);
 			IVecInt newLits = new VecInt();
 			for (Iterator<String> iterator = tmpLit.iterator(); iterator
 					.hasNext();)
@@ -270,13 +288,59 @@ public class OPBReader2007 extends OPBReader2006 {
 		return newVar;
 	}
 
-	private final Map<String, IVec<String>> productStore = new HashMap<String, IVec<String>>();
+	private final Map<String, IVec<String>> varToProduct = new HashMap<String, IVec<String>>();
+
+	private final Map<String, Map<String, String>> binaryProductToVar = new HashMap<String, Map<String, String>>();
 
 	private String getProductVariable(IVec<String> lits) {
-		for (Map.Entry<String, IVec<String>> c : productStore.entrySet())
+		if (lits.size() == 2) {
+			Map<String, String> map = binaryProductToVar.get(lits.get(0));
+			if (map == null)
+				return null;
+			return map.get(lits.get(1));
+		}
+		for (Map.Entry<String, IVec<String>> c : varToProduct.entrySet())
 			if (c.getValue().equals(lits))
 				return c.getKey();
 		return null;
 	}
 
+	@Override
+	public String decode(int[] model) {
+		StringBuffer stb = new StringBuffer();
+		int p;
+		for (int i = 0; i < model.length; i++) {
+			p = model[i];
+			if (Math.abs(p) <= nbVars) {
+				if (p < 0) {
+					stb.append("-x");
+					stb.append(-p);
+				} else {
+					stb.append("x");
+					stb.append(p);
+				}
+				stb.append(" ");
+			}
+		}
+		return stb.toString();
+	}
+
+	@Override
+	public void decode(int[] model, PrintWriter out) {
+		int p;
+		for (int i = 0; i < model.length; i++) {
+			p = model[i];
+			if (Math.abs(p) <= nbVars) {
+				if (model[i] < 0) {
+					out.print("-x");
+					out.print(-p);
+				} else {
+					out.print("x");
+					out.print(p);
+				}
+				out.print(" ");
+			}
+		}
+	}
+
 }
